/*
 * barycheb.h
 *
 *  Created on: May 16, 2014
 *      Author: skai
 */

#ifndef BARYCHEB_H_
#define BARYCHEB_H_
#define NULL 						0
#define PI  ( 3.14159265358979)

#define WINDOW_WIDTH  				131
#define WINDOW_HEIGHT  				80

#define Too_Few_Arguments 513	// #201 err number in HP
#define Bad_Argument_Type 514	// #202 err number in HP
#define Invalid_Syntax 262	    // #106 err number in HP
#define Positive_Underflow 769 	// #301 err number in HP
#define Negative_Underflow 770	// #302 err number in HP
#define Overflow 771			// #303 err number in HP
#define Undefined_Result 772  	// #304 err number in HP
#define Infinite_Result 773  	// #305 err number in HP
#define Invalid_Dimensions 1281	// #501 err number in HP



BOOL sat_push_double_vector(int elements,double *vector);
BOOL sat_push_double_cmatrix(int rows,int columns,double **rarray, double **iarray);
BOOL sat_push_double_cvector(int n,double *rvector, double *ivector);
double * sat_create_pop_double_vector(int *elements);
double ** sat_create_pop_double_matrix(int *m,int *n);
void	const_vector(int n, double r, double v[]);
double * copy_vector(double *cpy, double *orig,int n);

double ** create_matrix2(int m, int n);
double *  create_vector(int n);

void swap(double *x, double *y);

typedef struct {
	union {
		double r[5];
		struct {double xmin, xmax,dx, ymin,ymax;};
	};
} PLOTDATA, *PLOTDATA_PTR;

typedef struct {
	union {
		double r[2];
		struct {double x, y;};
	};
} CMP, *CMP_PTR;
CMP cmp_add(CMP a, CMP b);
CMP cmp_sub(CMP a, CMP b);
CMP cmp_mul(CMP a, CMP b);
CMP cmp_div(CMP a, CMP b);
CMP fcmp_mul(double a, CMP b);
CMP fcmp_XX_mul(double a, CMP b);
CMP fcmp_div(double a, CMP b);
CMP fcmp_add(double a, CMP b);

double fx(double x, double **data, double *t, int m );
int do_plot(double **data,double *t, int m, int nn, PLOTDATA plot);
double chebx(double x,double **data,int m );
double chebx1(double x,double **data,int m );
double *chebcoef ( double *c, int m);
double fbarydx(double x, double f, double **data, double *t, int m );

int forced_to_matrix=0;
int push_as_vector=0;
//hpg_t * HPG

BOOL
sat_push_double_vector(int n,double *vector){
	int j;
	SAT_OBJECT sat_array;
    sat_array=sat3_createrarray(ALLOC_TEMPOB,n,0);
    if(sat_array==0){ sat3_exiterror( SATERR_INSUFFMEMORY ); return FALSE;}
    for (j=0;j<n;j++) sat3_rarrayput_dbl( sat_array,j,0,vector[j]);
    sat3_push(sat_array);
    return TRUE;
}

void							/* create constant matrix of double */

const_matrix(int m, int n, double r, double **mat)
{
	int i,j;
	for (i = 0; i < m; i++){for (j=0;j<n;j++){ mat[i][j]=r;}}
}


void							/* create constant vector of double */
const_vector(int n, double r, double *v)
{
	int i;
	for (i = 0; i < n; i++)
		v[i] = r;
}


double *
create_vector(int m)
{
     // Dynamically allocate an n vector
	double r=0.0;
	double *vec;
	if(!(vec = (double *) calloc(m,sizeof(double)))){sat3_exiterror(SATERR_INSUFFMEMORY); return (0);}
	const_vector(m,r,vec); //	for (i=0;i<m;i++) {vec[i]=r;}
	return (vec);
}

double *
sat_create_pop_double_vector(int *n)
{
// double sat3_pop_dbl(0.0)
	SAT_OBJECT	eaddr;
int flag=0, rows=0,cols=0,j;
double *vector=0;

	if(sat3_depth()==0) sat3_exiterror(Too_Few_Arguments); //error: too few arguments
    eaddr= sat3_pick(1) ;
	if (!(flag=sat3_rarraysize(eaddr,&rows, &cols)))sat3_exiterror( Bad_Argument_Type );   //error: not an arry
	if ((cols))  sat3_exiterror( SATERR_INVALIDDIMENSION);
	if (!( vector=create_vector(rows))) sat3_exiterror(SATERR_INSUFFMEMORY);
	for (j=0;j<rows;j++) vector[j]=sat3_rarrayget_dbl( eaddr,j,0);
    sat3_drop(); //pop the vector from the stack
    *n=rows;
	return (vector);
}


double **
sat_create_pop_double_matrix(int *m,int *n)
{
// double sat3_pop_dbl(0.0)
SAT_OBJECT	eaddr;
int cols=0, rows=0,j;
int flag,i;
double **arry;
    forced_to_matrix=0;
	if(sat3_depth()==0) sat3_exiterror(Too_Few_Arguments); //error: too few arguments
	eaddr= sat3_pick(1) ;
	if (!(flag=sat3_rarraysize(eaddr,&rows, &cols)))sat3_exiterror( Bad_Argument_Type );   //error: not an arry
	if(!(cols)) {cols=1;forced_to_matrix=1;}
	if (!( arry=create_matrix2(rows,cols))) sat3_exiterror(SATERR_INSUFFMEMORY );

for (i=0;i<rows;i++){
			for (j=0;j<cols;j++){
				arry[i][j]=sat3_rarrayget_dbl( eaddr,i,j);
			}
		}

    sat3_drop(); //pop the arry from the stack
    *m=rows;
	*n=cols;
	return arry;

}



double **
create_matrix2(int m, int n)
{
     // Dynamically allocate an mxn
     //

	int i;
	const double r=0.0;
	double  **a;
    if(!(a = (double **)calloc(m,sizeof(double *)))) { sat3_exiterror(SATERR_INSUFFMEMORY);  return (0);}
    for (i=0;i<m;i++) {
		if (!(a[i] = (double *)calloc(n,sizeof(double)))){a[0]=0;sat3_exiterror(SATERR_INSUFFMEMORY); return (0);}
	}
	const_matrix(m,n,r,a); //	for (i=0;i<m;i++){for(j=0;j<n;j++){a[i][j]=r;}}
	return a;
}
 void
free_matrix2(int m,int n,double **a)    // actually only a list of separate vectors ***
{	// free matrix generated by create_matrix2
	int i;
	n=0;   // avoid warning
	for (i=0;i<m;i++){free(a[i]);}
	free(a);
}


 CMP cmp_add(CMP a, CMP b){
	 CMP z;
	 z.x=a.x+b.x;
	 z.y=a.y+b.y;
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, a.x,a.y));
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, b.x,b.y));
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, z.x,z.y));
	 return z;
 }
 CMP fcmp_add(double a, CMP b){
	 CMP z;
	 z.x=a+b.x;
	 z.y=b.y;
	 return z;
 }
 CMP cmp_sub(CMP a, CMP b){
	 CMP z;
	 z.x=a.x-b.x;
	 z.y=a.y-b.y;
	 return z;
 }
 CMP cmp_mul(CMP a, CMP b){
	 CMP z;
	 z.x=a.x*b.x-a.y*b.y;
	 z.y=a.x*b.y+a.y*b.x;
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, a.x,a.y));
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, b.x,b.y));
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, z.x,z.y));
	 return z;
 }
 CMP cmp_XX_mul(CMP a, CMP b){
	 CMP z;
	 z.x=a.x*b.x;
	 z.y=a.x*b.y+a.y*b.x;
	 return z;
 }
 CMP fcmp_mul(double a, CMP b){
	 CMP z;
	 z.x=a*b.x;
	 z.y=a*b.y;
	 return z;
 }
 CMP cmp_div(CMP a, CMP b){
	 double m;
	 CMP z;
	 m=1.0/(b.x*b.x+b.y*b.y);
	 z.x=(a.x*b.x+a.y*b.y)*m;
	 z.y=(a.y*b.x-a.x*b.y)*m;
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, a.x,a.y));
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, b.x,b.y));
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, z.x,z.y));
	 return z;
 }
 CMP fcmp_div(double a, CMP b){
	 double m;
	 CMP z;
	 m=1.0/(b.x*b.x+b.y*b.y);
	 z.x=a*b.x*m;
	 z.y=(-a*b.y)*m;
//	 sat3_push_dbl_real(a);
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, b.x,b.y));
//	 sat3_push(sat3_createcplx_dbl(ALLOC_HEAP, z.x,z.y));
	 return z;
 }
double * copy_vector(double *cpy, double *orig,int n){
	int k;
	for(k=0;k<n;k++) cpy[k]=orig[k];
	return cpy;
}



#endif /* BARYCHEB_H_ */
