/*
 * davws.c
 *
 *  Created on: Sep 17, 2014
 *      Author: skai
 */
#include <hpgcc3.h>

#include "chebroots.h"


/*
! **********************************************
!  Last updated: December 10, 2012
! **********************************************
!
! This subroutine computes the roots of a real polynomial of degree N
! in any recursively defined polynomial basis. The general
! algorithm is outlined in:
!
! Fast Computation of Eigenvalues of Companion, Comrade and Related Matrices, BIT Numerical Mathematics 2012
!
! **************************************************************
!
! Input variables:
!
! BANDSWITCH   0 indicates no COEFFS are input, 1 indicates COEFFS are given
!
! N            degree of polynomial
!
! K            maximum length of the recursion == 3 for chebyshev
!
! POLY         array of dimension N+1 containing the coefficients
!              of the polynomial in terms of the basis elements
!
! COEFFS       array of dimension (N x K) containing the recursion
!	       coefficients of the basis elements, ignored if BANDSWITCH = 0
!
! Output variables:
!
! RROOTS       contains the real part of the computed roots, dimension N
!
! IROOTS       contains the imaginary part of the computed roots, dimension N
!
! ITERATIONS   integer array storing the number of iterations per root calculation
!
! FLAG         reports the position of any instance where the algorithm
!              tried to divide by 0
!
! ***************************************************************

*/



// subroutine DAVW2(BANDSWITCH,N,K,POLY,COEFFS,RROOTS,IROOTS,ITERATIONS,FLAG)
 int DAVW2(int BANDSWITCH,int N,int K,double *POLY, double **COEFFS,double *RROOTS,double *IROOTS,int *ITERATIONS)
 {



//	implicit none

//	integer, intent(in) :: BANDSWITCH,N,K
//	integer, intent(inout) :: ITERATIONS(N),FLAG
//	double precision, intent(in) :: POLY(N+1),COEFFS(N,K)
//	double precision, intent(inout) :: RROOTS(N),IROOTS(N)

	 int FLAG = 0;

//	if(BANDSWITCH == 0) call DLR(N,POLY,RROOTS,IROOTS,ITERATIONS,FLAG);

//	else if(BANDSWITCH == 1)then

	 FLAG=DBLR(N,K,POLY,COEFFS,RROOTS,IROOTS,ITERATIONS,0,0);

//	else

//		write(*,*) "Not a valid argument for BANDSWITCH"
//		write(*,*) ""
//		return
//	end if
//end subroutine
	 return FLAG;
}

 /*
! **************************************************************
!
! This subroutine calculates the roots using any congenial matrix
! NOTE skai:  K is assumed 3
! **************************************************************
*/
// subroutine DBLR(N,K,POLY,COEFFS,RROOTS,IROOTS,ITERATIONS,FLAG,INIT,CHASE)

int DBLR(int N,int K,double *poly,double **Coeffs,double *rroots,double *iroots,int *iterations,
 		  int init,int chase){

//	implicit none

//	integer, intent(in) :: N,K,INIT,CHASE
//	double precision, intent(in) :: POLY(N+1),COEFFS(N,K)
//	double precision, intent(inout) :: RROOTS(N),IROOTS(N)
//	integer, intent(out) :: ITERATIONS(N),flag



//	integer :: stop_index,start_index,zero_index,ii,jj,hh,mm,it_count=0,it_max
//	integer :: length,switch=0
//	double precision :: tolerance=1d-16,zero_point,diagonals,diag1,diag2
//	double precision :: L1,L2,L3,W[2][2],A(3,2),M,bulge,trace,detm,G(3)
//	double precision, allocatable :: spike(:),Blocks(:,:,:),Bands(:,:)
	int i;
	int  stop_index,start_index,zero_index,ii,hh,mm,it_count=0,it_max;
	int length,/*switch_flag=0,*/ flag=0;
	int warn_exceptional_shift_dblr=FALSE;
	double tolerance=1e-16,zero_point,diagonals,diag1,diag2;
	//	double L1,L2,L3,W[2][2],A[3][2],M,bulge,trace,detm,G[3];
	double A11,A12,A21,A22,A31,A32;
	double a11,a12,a21,a22,a31,a32;
	double W11,W12,W21,W22;
	double L[4],/*M,bulge,*/trace,detm,G[4]; // increased every dimension by one
//	double precision, allocatable :: spike(:),Blocks(:,:,:),Bands(:,:)
	double *spike, *b_11, *b_12, *b_21, *b_22, **Bands;


// ! Initialize Bands and Blocks

//	allocate(spike(N),b_22[N),Bands(N,K))
	spike=create_vector(N+1);		//increased each dimension by 1
	b_11=create_vector(N+1);	//increased each dimension by 1
	b_12=create_vector(N+1);	//increased each dimension by 1
	b_21=create_vector(N+1);	//increased each dimension by 1
	b_22=create_vector(N+1);	//increased each dimension by 1
	Bands=create_matrix2_f(N,K);
//	Bands=create_matrix2(N+1,K+1); 	//increased each dimension by 1

	if(fabs(poly[1]) == 0 || fabs(Coeffs[1][1]) == 0){
		sat3_push_string("Polynomial is of degree less than N!");
		sat3_exiterror(SATERR_INVALIDARRAYELEM);
		return(flag=1);
	}

	//	spike = -Poly(2:(N+1))/Poly(1)*Coeffs(1,1)
	for (i=1;i<=N;i++) spike[i] = -poly[i+1]/poly[1]*Coeffs[1][1];


	//	spike(1:(K-1)) = spike(1:(K-1)) + Coeffs(1,2:K)
	spike[1]+= Coeffs[1][2];
	spike[2]+= Coeffs[1][3];

//		const_matrix(Np1,Kp1,0.0,Bands); // increased by 1,1

	//	Bands(1:(N-1),1:K) = Coeffs(2:N,1:K)
	for (i=1;i<=N-1;i++){
		Bands[i][1]= Coeffs[i+1][1];
		Bands[i][2]= Coeffs[i+1][2];
		Bands[i][3]= Coeffs[i+1][3];
	}
	Bands[N][1]= 0.0;	// skai2
	Bands[N][2]= 0.0;	// skai2
	Bands[N][3]= 0.0;   // skai2


//	Blocks(:,:,1) = 0d0
//	b_22[1) = 1d0
	b_11[1]=0.0;
	b_12[1]=0.0;
	b_21[1]=0.0;
	b_22[1]=1.0;

//	do ii=1,(N-1)
//        printf ("\n");
	for (ii=1;ii<=N-1;ii++){
//		printf("Band  %d %f %f %f ",ii,Bands[ii][1],Bands[ii][2],Bands[ii][3]);
		if(fabs(Bands[ii][1]) == 0){
			sat3_push_string("Matrix is not properly upper-hessenberg!");
			sat3_exiterror(SATERR_INVALIDDEFINITION);
			flag = 2;
			return(2);
		}

		L[1] = spike[ii]/Bands[ii][1];

		if (N+1-ii<K) length = N+1-ii; else length=K;
//		spike(ii:(ii+length-1)) = spike(ii:(ii+length-1)) - L1*Bands(ii,1:length);
		for (i=1;i<=length;i++) spike[ii+i-1] = spike[ii+i-1] - L[1]*Bands[ii][i];

		b_11[ii+1] = L[1];
		b_12[ii+1] = 1.0;
		b_21[ii+1] = 1.0;
		b_22[ii+1] = 0.0;
	};// end do

	Bands[N][1] = spike[N];
//! Main loop

  stop_index = N;
  start_index = 2;
  zero_index = 1;
  it_max = 20*N;
  it_count = 0;
  flag = 0;

  for(mm=1;mm<=it_max;mm++){ //do mm=1,it_max

//	! Zero check
	for (ii=1;ii<=stop_index;ii++){ // do ii=1,stop_index{
		zero_point = fabs(b_21[stop_index+1-ii]);

		if(ii>1)diag2 = fabs(b_11[stop_index+2-ii]*b_22[stop_index+1-ii]*Bands[stop_index+1-ii][1]);
		else diag2 = fabs(b_22[stop_index+1-ii]*Bands[stop_index+1-ii][1]);


		if(ii<stop_index) diag1 = fabs(b_11[stop_index+1-ii]*b_22[stop_index-ii]*Bands[stop_index-ii][1]);
		else diag1 = fabs(b_11[stop_index+1-ii]*Bands[stop_index+1-ii][1]); // skai


		diagonals = diag1 + diag2;

		if(zero_point < tolerance*diagonals){
			zero_index = stop_index+1-ii;
			b_21[zero_index] = 0.0;

//			! Deflate down if not at bottom
			if(zero_index<stop_index){
				b_11[zero_index+1] = b_11[zero_index+1]*b_22[zero_index];
				b_12[zero_index+1] = b_12[zero_index+1]*b_22[zero_index];
				b_12[zero_index] = b_12[zero_index]/b_22[zero_index];
				b_22[zero_index] = 1.0;
				start_index = zero_index+1;
			}

			break; // goto ferdigAA;

		}
 }
//	ferdigAA: // todo: check if goto-ferdig  correct

//	! zero at bottom
	if(zero_index==stop_index){
		rroots[zero_index] = b_22[zero_index]*Bands[zero_index][1];
		iroots[zero_index] = 0.0;

		iterations[N-zero_index+1] = it_count;
		it_count = 0;

		if(stop_index<=1){ break; /*goto ferdigBB;	*/ } // todo check

//		! deflate up
//		Blocks(:,2,zero_index-1) = b_11[stop_index]*Blocks(:,2,zero_index-1);
		b_12[zero_index-1] = b_11[zero_index]*b_12[zero_index-1];
		b_22[zero_index-1] = b_11[zero_index]*b_22[zero_index-1];
		b_11[zero_index] = 1.0;

		stop_index = stop_index-1;
 }
//	! 2x2 case
	else if(stop_index==start_index){
//		! calculate eigenvalues using modified quadratic
		if(K == 1){
		W11 = b_11[stop_index]*Bands[stop_index-1][1];
		W12 = b_12[stop_index]*Bands[stop_index][1];
		W21 = b_21[stop_index]*Bands[stop_index-1][1];
		W22 = b_22[stop_index]*Bands[stop_index][1];
 	 	 }
 	 else {
		W11 = b_11[stop_index]*Bands[stop_index-1][1];
		W12 = b_11[stop_index]*Bands[stop_index-1][2] + b_12[stop_index]*Bands[stop_index][1];
		W21 = b_21[stop_index]*Bands[stop_index-1][1];
		W22 = b_21[stop_index]*Bands[stop_index-1][2] + b_22[stop_index]*Bands[stop_index][1];
		}

		trace = W11 + W22;
		detm = W11*W22 - W12*W21;

		if(trace*trace < 4.0*detm){
			rroots[stop_index] = trace/2.0;
			iroots[stop_index] = fsqrt(4.0*detm - trace*trace)/2.0;
			rroots[stop_index-1] = trace/2.0;
			iroots[stop_index-1] = -fsqrt(4.0*detm - trace*trace)/2.0;
 	 	 }
		else
		{
			rroots[stop_index] = (trace + fsqrt(trace*trace - 4.0*detm))/2.0;
			iroots[stop_index] = 0.0;
			rroots[stop_index-1] = (trace - fsqrt(trace*trace - 4.0*detm))/2.0;
			iroots[stop_index-1] = 0.0;
		}

		iterations[N-stop_index+1] = it_count;
		iterations[N-stop_index+2] = 0;
		it_count = 0;

		stop_index = stop_index-2;

//		! Breakout when done
		if(stop_index==0) break; // goto ferdigCC; // todo check if correct

//		! Deflate up
		b_12[stop_index] = b_12[stop_index]*b_11[stop_index+1];
		b_22[stop_index] = b_22[stop_index]*b_11[stop_index+1];

		b_11[stop_index+1] = 1.0;

//	! more than one block
	}else{
		it_count = it_count + 1;

//		! shift calc
		if(K == 1){
		W11 = b_11[stop_index]*Bands[stop_index-1][1]*b_22[stop_index-1];
		W12 = b_12[stop_index]*Bands[stop_index][1];
		W12 = W12*b_22[stop_index-1];
		W21 = b_21[stop_index]*Bands[stop_index-1][1];
		W22 = b_22[stop_index]*Bands[stop_index][1];
	}
		else{
		W11 = b_11[stop_index]*Bands[stop_index-1][1]*b_22[stop_index-1];
		W12 = b_11[stop_index]*Bands[stop_index-1][2] + b_12[stop_index]*Bands[stop_index][1];
		W12 = W12*b_22[stop_index-1];
		W21 = b_21[stop_index]*Bands[stop_index-1][1];
		W22 = b_21[stop_index]*Bands[stop_index-1][2] + b_22[stop_index]*Bands[stop_index][1];
	}

		if(mm == 0){
			trace = 0.0;
			detm = -1.0;}
		else if(!(it_count%15)){  // sjekk
			dnormalpoly(G,1,2);
			G[2] = G[1]*G[1] + G[2]*G[2];
			G[2] = fsqrt(G[2]);
			trace = 2.0*G[1]/G[2];
			detm = 1.0;
//			sat3_push_string("Exceptional shift in DBLR!");
			warn_exceptional_shift_dblr=TRUE;
		}else{
			trace = W11 + W22;
			detm = W11*W22 - W12*W21;
		}

//		! Build
		A11=A12=0.0;
		A21=A22=0.0;
		A31=A32=0.0;
		if(K == 1){
			A11 = Bands[start_index-1][1];
			A22 = Bands[start_index][1];
		}
		else
		{
			A11 = Bands[start_index-1][1];
			A12 = Bands[start_index-1][2];
			A22 = Bands[start_index][1];
		}

//		A(2:3,:) = matmul(Blocks(:,:,start_index+1),A(2:3,:));
		i=start_index;
		a21=A21; a22=A22;
		a31=A31; a32=A32;
		A21=b_11[i+1]*a21 + b_12[i+1]*a31;  A22=b_11[i+1]*a22 + b_12[i+1]*a32;
		A31=b_21[i+1]*a21 + b_22[i+1]*a31;  A32=b_21[i+1]*a22 + b_22[i+1]*a32;





//		A(1:2,:) = matmul(Blocks(:,:,start_index),A(1:2,:));
		a11=A11; a12=A12;
		a21=A21; a22=A22;
		A11=b_11[i]*a11 + b_12[i]*a21;  A12=b_11[i]*a12 + b_12[i]*a22;
		A21=b_21[i]*a11 + b_22[i]*a21;  A22=b_21[i]*a12 + b_22[i]*a22;



		G[1] = A11*A11 - trace*A11 + detm + A12*A21;
		G[2] = A21*(A11 + A22 - trace);
		G[3] = A21*A32;

//		! Build L1 L2 and L3
		L[2] = G[3]/G[2];
		L[3] = G[2]/G[1];

//		! Build bulge
		flag=DBB(L,start_index,N,b_11, b_12,b_21,b_22,init);

//		! Move L2 through bands
		DBP(L,2,start_index+1,start_index,stop_index,N,K,Bands);

//		! Move L3 through bands
		DBP(L,3,start_index,start_index,stop_index,N,K,Bands);

//		! Turnover iterations
		if(chase == 0){
//		do hh=(start_index+2),(stop_index)
			for(hh=(start_index+2);hh<=(stop_index);hh++){
//			! Pass through Blocks and turnover L's over
			flag=DGATO(b_11, b_12, b_21, b_22,hh-1,hh,L,2);
			flag=DGATO(b_11, b_12, b_21, b_22,hh-2,hh-1,L,3);

			flag=DGLTO(L);

//			! Move L2 through bands
			DBP(L,2,hh,start_index,stop_index,N,K,Bands);

//			! Move L3 through bands
			DBP(L,3,hh-1,start_index,stop_index,N,K,Bands);

			}//end do

//		! Dump the Bulge
//		Blocks(:,1,stop_index) = Blocks(:,1,stop_index) + L2*Blocks(:,2,stop_index);
		b_11[stop_index] = b_11[stop_index] + L[2]*b_12[stop_index];
		b_21[stop_index] = b_21[stop_index] + L[2]*b_22[stop_index];

		flag=DGATO(b_11, b_12, b_21, b_22,stop_index-1,stop_index,L,3);

//		! Move L1 through bands
		DBP(L,1,stop_index,start_index,stop_index,N,K,Bands);

//		Blocks(:,1,stop_index) = Blocks(:,1,stop_index) + L[1]*Blocks(:,2,stop_index);
		b_11[stop_index] = b_11[stop_index] + L[1]*b_12[stop_index];
		b_21[stop_index] = b_21[stop_index] + L[1]*b_22[stop_index];

//		! Move L3 through bands
		DBP(L,3,stop_index,start_index,stop_index,N,K,Bands);

//		Blocks(:,1,stop_index) = Blocks(:,1,stop_index) + L[3]*Blocks(:,2,stop_index);
		b_11[stop_index] = b_11[stop_index] + L[3]*b_12[stop_index];
		b_21[stop_index] = b_21[stop_index] + L[3]*b_22[stop_index];
	}
		else if(chase == 1){
		for(hh=start_index+2;hh<=stop_index;hh++){//do hh=(start_index+2),(stop_index)
//			! Pass through Blocks and turnover L's over
			flag=DGATO(b_11, b_12, b_21, b_22,hh-1,hh,L,2);
			flag=DGATO(b_11, b_12, b_21, b_22,hh-2,hh-1,L,3);

//			! Move L1 through bands
			DBP(L,1,hh-1,start_index,stop_index,N,K,Bands);

//			! Move L2 through bands
			DBP(L,2,hh,start_index,stop_index,N,K,Bands);

//			! Move L3 through bands
			DBP(L,3,hh-1,start_index,stop_index,N,K,Bands);

//			! Pass L1 through Blocks
			flag=DGATO(b_11, b_12, b_21, b_22,hh-1,hh,L,1);

		}//end do

//		! Dump the Bulge
//		Blocks(:,1,stop_index) = Blocks(:,1,stop_index) + L2*Blocks(:,2,stop_index);
		b_11[stop_index] = b_11[stop_index] + L[2]*b_12[stop_index];
		b_21[stop_index] = b_21[stop_index] + L[2]*b_22[stop_index];

//		flag=DGATO(Blocks(:,:,stop_index-1),Blocks(:,:,stop_index),L,3);
		flag=DGATO(b_11, b_12,b_21,b_22,stop_index-1,stop_index,L,3);

//		! Move L1 through bands
		DBP(L,1,stop_index,start_index,stop_index,N,K,Bands);

//		Blocks(:,1,stop_index) = Blocks(:,1,stop_index) + L1*Blocks(:,2,stop_index);
		b_11[stop_index] = b_11[stop_index] + L[1]*b_12[stop_index];
		b_21[stop_index] = b_21[stop_index] + L[1]*b_22[stop_index];

//		! Move L3 through bands
		DBP(L,3,stop_index,start_index,stop_index,N,K,Bands);

//		Blocks(:,1,stop_index) = Blocks(:,1,stop_index) + L3*Blocks(:,2,stop_index);
		b_11[stop_index] = b_11[stop_index] + L[3]*b_12[stop_index];
		b_21[stop_index] = b_21[stop_index] + L[3]*b_22[stop_index];

		}// end if
	}//end if

 }// end do

//  ! set flag for maxing out loop
  if(mm == it_max) flag = 1;
  if (warn_exceptional_shift_dblr)flag+=10;

//	! free memory
//	deallocate(spike,Blocks,Bands)
free_matrix2_f(N,K,Bands);
free(b_11);free(b_12);free(b_21);free(b_22);
free(spike);free(iterations);
// end subroutine
return (flag);
 }
/*
! **********************************************************************
!
!
!
! **********************************************************************

subroutine DGATO(A,B,L,flag)
*/

// int DGATO(double A[3][3], double B[3][3],double L[4],int LL){
int DGATO(double *r11,double *r12, double *r21, double *r22,
		  int A, int B, double L[4],int LL){
//implicit none

// integer :: flag
//double precision :: A[2][2],B[2][2],L
//
//	A(:,1) = A(:,1) + L*B[1][1]*A(:,2);
//	 A[1][1] = A[1][1] + L[LL]*B[1][1]*A[1][2];
//	 A[2][1] = A[2][1] + L[LL]*B[1][1]*A[2][2];
//
	 r11[A] = r11[A] + L[LL]*r11[B]*r12[A];
	 r21[A] = r21[A] + L[LL]*r11[B]*r22[A];

	//	if(fabs(A[2][1]) == 0){return(1);}
	if(fabs(r21[A]) == 0){ sat3_exiterror(SATERR_UNDEFINEDRESULT);/* return(1); */ } // skai 2014-12-30


	//	L[LL] = L[LL]*B[2][1]/A[2][1];
		L[LL] = L[LL]*r21[B]/r21[A];

//	B(2,:) = B(2,:) - L*A[2][2]*B(1,:);
	 r21[B] = r21[B] - L[LL]*r22[A]*r11[B];
	 r22[B] = r22[B] - L[LL]*r22[A]*r12[B];

//end subroutine
	return(0);}

/*
! **********************************************************************
!
!
!
! **********************************************************************

subroutine DGLTO(L1,L2,L3,flag)
*/
int DGLTO(double *L)
{
// implicit none


double temp1,temp2,temp3;

	temp1 = L[1];
	temp2 = L[2];
	temp3 = L[3];

	L[3] = temp1 + temp3;

	if(fabs(L[3]) == 0){
		sat3_exiterror(SATERR_UNDEFINEDRESULT);
/*		sat3_push_string( "Tried to divide by 0. Algorithm failed!"); */ // skai 2014-12-30
		return(1);
	}

	L[2] = temp2*temp3/L[3];
	L[1] = temp2 - L[2];

// end subroutine
return (0);
}

/*
! **********************************************************************
!
! Double Precision Band Pass
!
! **********************************************************************

subroutine DBP(L,start,top,bottom,N,K,Bands)
*/
void DBP(double L[4], int LL,int start, int top, int bottom, int N, int K, double **Bands){
//implicit none

// integer :: start,top,bottom,N,K,jj,length
	int jj, length;
//	double precision :: L,Bands(N,K),bulge
	double bulge;
		if (start-top+1<K-1) length = start-top+1; else length =K-1;
		for (jj=1; jj<=length;jj++){ // do jj=1,length
			Bands[start-jj][jj] = Bands[start-jj][jj] + L[LL]*Bands[start-jj][jj+1];
		}

		bulge = L[LL]*Bands[start][1];

		L[LL] = bulge/Bands[start-1][1];

		if (K-1<bottom+1-start)length = K-1; else length= bottom+1-start;
		for(jj=1;jj<=length;jj++){ //do jj=1,length
			Bands[start][jj] = Bands[start][jj] - L[LL]*Bands[start-1][jj+1];
		}

// end subroutine
			return;
}




/*
! **********************************************************************
!
! Double Precision Bulge Build
!
! **********************************************************************

subroutine DBB(L1,L2,L3,start,N,Blocks,switch,flag)
*/
int DBB(double L[4], int start, int N,
		double *b_11,  double *b_12,  double *b_21,  double *b_22,
		int switch_flag){
 //implicit none

// integer :: start,N,switch,flag
// double precision :: L1,L2,L3,b_22[N),M,W[2][2]
	int flag=0;
	double M,W21,W22;



		if(switch_flag == 0){
//			W11 = b_11[start];
//			W12 = b_12[start];
			W21 = b_21[start];
			W22 = b_22[start];
//			Blocks(2,:,start) = Blocks(2,:,start) - L3*Blocks(1,:,start);
			b_21[start] = b_21[start] - L[3]*b_11[start];
			b_22[start] = b_22[start] - L[3]*b_12[start];

			if(fabs(b_21[start]) == 0){sat3_exiterror(SATERR_UNDEFINEDRESULT); return(1);} // skai 2014-12-30

			L[1] = -L[2]*W21/b_21[start];
			M = -L[2]*W22 - L[1]*b_22[start];

//			Blocks(2,:,start+1) = Blocks(2,:,start+1) + M*Blocks(1,:,start+1);
			b_21[start+1] = b_21[start+1] + M*b_11[start+1];
			b_22[start+1] = b_22[start+1] + M*b_12[start+1];
		}
		else if(switch_flag == 1){
//			Blocks(2,:,start+1) = Blocks(2,:,start+1) - L2*b_22[start)*Blocks(1,:,start+1);
			b_21[start+1] = b_21[start+1] - L[2]*b_22[start]*b_11[start+1];
			b_22[start+1] = b_22[start+1] - L[2]*b_22[start]*b_12[start+1];

			if(fabs(b_21[start+1]) == 0){sat3_exiterror(SATERR_UNDEFINEDRESULT);return(1);} // skai 2014-12-30

			L[1] = -L[2]*b_21[start]/b_21[start+1];

//			Blocks(:,1,start)= Blocks(:,1,start) - L1*Blocks(1,1,start+1)*Blocks(:,2,start);
			b_11[start]= b_21[start] - L[1]*b_11[start+1]*b_12[start];
			b_21[start]= b_21[start] - L[1]*b_11[start+1]*b_22[start];

//			Blocks(2,:,start) = Blocks(2,:,start) - L3*Blocks(1,:,start);
			b_21[start] = b_21[start] - L[3]*b_11[start];
			b_22[start] = b_22[start] - L[3]*b_12[start];

//			flag= DGATO(Blocks(:,:,start),Blocks(:,:,start+1),L,1);
			flag= DGATO(b_11, b_12, b_21, b_22,start,start+1,L,1);

		}

//end subroutine
		return(flag);}


/*
! **********************************************************************
!
!
!
! **********************************************************************

subroutine dnormalpoly(N,poly)
*/
 void dnormalpoly(double *poly, int start, int stop){
//	implicit none

//	integer, intent(in) :: N
//	real(kind(1d0)), intent(inout) :: poly(N)

	double  u,v,s,pi = 3.141592653589793239;
	int ii,jj;


	for ( ii=start;ii<=stop;ii++){
		for (jj=1;jj<=20;jj++){
			u=rand();u =u/RAND_MAX;
			v=rand(); v=v/RAND_MAX;
			s = u*u + v*v;
			if((s>0) && (s < 1)){
				poly[ii] = cos(2.0*pi*v)*sqrt(-2.0*log(u));
				break; //goto fini;
			}
		}
// fini: ;
		}
// end subroutine
return;
}

/*
! **********************************************************************
!
!
!
! **********************************************************************

SUBROUTINE init_random_seed()

	implicit none

        INTEGER :: ii, kk, clock
        INTEGER, DIMENSION(:), ALLOCATABLE :: seed

        CALL RANDOM_SEED(size = kk)
        ALLOCATE(seed(kk))

        CALL SYSTEM_CLOCK(COUNT=clock)

        seed = clock + 37 * (/ (ii - 1, ii = 1, kk) /)
        CALL RANDOM_SEED(PUT = seed)

	DEALLOCATE(seed)
END SUBROUTINE

 *
 */

