/*
 * realchebyshev.c
 *  pop a vector of chebyshev coeficient, pushes its roots
 *  uses eigenvalues from companion (implicit) matrix
 *  port from fortran90 by Jared Aurentz
 *  www.math.wsu.edu/students/jaurentz/
 *  Created on: Sep 7, 2014
 *      Author: skai
 */

#include <hpgcc3.h>
// #include <hpgraphics.h>

// ARMCODE_CONFIG_STACK_STANDARD
// TO KEEP THE KEYBOARD AND TIMERS ENABLED, USE INSTEAD:
// ARMCODE_CONFIG(_STACK_SPEED_120MHZ);
ARMCODE_CONFIG(startup)= /*_NO_SCREEN  |  _NO_WAITKEY |_NO_KEYBOARD| */ _SPEED_120MHZ;
ARMCODE_CONFIG(memory_req)=USE_MAINRAM|USE_ERAM|USE_KOSRAM|RESERVE_TEMPOB(16)|RESERVE_USEROB(16);

int forced_to_matrix=0;
int push_as_vector=0;
int debug=1;
int  mode;
int hpdigits;

#include "chebroots.h"

double * der1(int nm1, double * a, int bigrange, double aa,double bb){ // TODO: verify
// used by cos transform (due to task 5)
	// ref aaabook 9.18
	// nm1:  a[0..n]  n dim of a, nm1=n-1,
	int i;
	double t0,t1=0.0,t2=0.0, r;
	r=a[nm1];
	a[nm1]=0.0;
	 for(i=nm1;i>0;i--){
		 t0=r*i*2+t2; //
		 t2=t1;t1=t0;
		 r=a[i-1];
		 a[i-1]=t0;
	 }
	    a[0]*=.5; // check this in connection to bigrange
//    if(bigrange) scalevec(a,2.0/(bb-aa), 0,nm1);
	return a;
}


int cheb_balance(double balanse_factor, double *T, int n_in, double *r){
//cheb_balance(input_flag,poly_input, n_in, &r);
// n_in : number of elements, size of T
	//	balanse_factor=1.2; // comment 1.2 seems to be a good value at least for symmetric systems...
	/*
	 * balance consist of effectively adding a root. We will add a root outside of interval [ -1..1],
	 *  which will be easy to remove later.
	 *  We can do this by resampling the original function,
	 *  alternatively
	 *  by transforming the chebyshev coeffisients to real x-axis, multiply with
	 *  the (x[i]-root) and  back to chebyshev
	 *  in practical terms
	 *  or transform  chebyshev and the vector [root, 1] to real x-axis,
	 *  then pairwise multiplication in x-axis and transfer back again.
	 *  The last option is most fancy, we will use this.
	 *  Note we must increase size of the arrays to 2^n+1 for the FFT based convolution to work
	 */
	/* do some smart things here */
	int i,n=0, balanse_required;
	const double eps=1E-14;
	double c0,st0, st1, st2;

//	clear_screen();
//	printf("n_in %d \n", n_in); WAIT_CANCEL;
/*  always discard any trailing cheb coeffisients*/
	i=n_in-1;while((fabs(T[i])<eps)&&i)i--;
	n=i+1; // size of coeffisients in use
//	sat_push_double_vector(n,T);
//	printf("n in use %d \n", n); WAIT_CANCEL;

	if(!(i)) sat3_exiterror(SATERR_INVALIDDEFINITION); // all Ti , i=n..1 is 0.0 or less than eps or [a, ..] i.e. only constant

	//	if (n==2)  {root= T[0]/T[1];T[0]=root;sat3_push_dbl_real(root);goto ferdig;}
/*  balance necessary if T[n-1-1] ~=0,  */
	if(fabs(T[n-2])<eps) balanse_required=TRUE;
	if (balanse_required || (fabs(*r) > 1e-6)) {
/*		optimum_scale depends on algorithm, assumed that we want to have T[n-2] ~T[n-1] */
//		cn_2 = T[n-3]*.5 + sc*T[n-2];
//		cn_1 = T[n-2]*.5;
//  sc=		(T[n-2]*.5 - T[n-3]*.5)/T[n-2] = 0,5(1-T[n-3]/T[n-2])
// if (fabs(sc) < 1.2 )	sc = 1.2*sign(sc);  // require that the root is outside of interesting interval [-1..1]
//  balanse_factor=sc    // we have now included a root
//  problem : if sc already is a root.....
	n++;   // increase T and make sure it is 0.0
	T[n-1]=0.0;
	c0=T[0];
	st0=0.0;
	st1=T[0];
	for (i=0;i<(n-1);i++){
		st2=T[i+1];
		T[i]=(st0+st2)/2+balanse_factor*st1;
		st0=st1;
		st1=st2;
	}
	T[n-1]=st0/2;
	T[1] += c0/2;

//// transfer to real x-axis
//m=1;
//while (m<n) m*=2;m++; // i is now 2^n+1, required size for CDT
//
////printf(" m cheb %d\n", m); WAIT_CANCEL;
//tmp=create_vector(m); tmp[0]=-balanse_factor; tmp[1]=1.0;
//
//T[0]*=2.0;T[m-1]*=2.0; // special for DCT definition
//fftini(T,m,5);
//
//tmp[0]*=2.0; // special for DCT definition
//fftini(tmp,m,5);
//for (j=0;j<m;j++)T[j]*=tmp[j];
//free(tmp);
//
//// transfer back
//fftini(T, m, -5);
//T[0]*=0.5; T[m-1]*=0.5;
//n++;
}
// n is required size, is available size(from input)
//sat_push_double_vector(n,T);
*r=T[n-1];
//printf("  min size n %d\n",n); WAIT_CANCEL;

return (n);
}

int rescheck_skai( double *wr, double *wi, double *cheb0, double*cheb1, int n, double ab_rms[2],int NR_improve, int Allroots){
/* n is number of roots, and n+1   is size of cheb0, cheb1
 NR_improve: =1 improve roots with Newton-Rhapson when possible
 Allroots is flag, determines
    FALSE: roots are to be selected within [0,0] and only real roots accepted, then NR on the real root
    TRUE: do nothing ( todo: use N-R to improve the complex root), do not calculate rms
 cheb0 is reversed
 cheb1 is used as scratch
 limits:
  imaginary part of |x| < eps_bad
  real part of |x| < eps_real=1+1E-6
  f(x0) < eps_bad
 on output:
  wr is accepted roots
  wi is accepted roots improved by NR whenever possible
  cheb1[0]=rms(cheb[wr])
  cheb1[1]=rms(cheb[wi])
  nreal_roots is number of real roots or all_roots if set

*/
	int m=0,i,offset, nrealroots=0;

	double eps=1E-12, eps_real=1+1E-6,eps_real_m=-1-1E-6, eps_bad=1E-6,eps_bad_m=-1E-6;
	double rms=0.0, rms_nr=0.0, dx,x,x_nr, f0,f1, a,b,r0,r1;
//	double *err,*err_nr;
//	err=create_vector(n);
//	err_nr=create_vector(n);
// reverse cheb1 from f90
//	clear_screen();
copy_vector(cheb1, cheb0,n+1);
//	for(i=0;i<n+1;i++)cheb1[i]=cheb0[n-i];
//	copy_vector(cheb0, cheb1,n+1);
	der1(n, cheb1, 0, -1.0, 1.0); // derivate of chebyshev polynom for interval [-1..1]
//	printf("n+1 %d \nT[n-1] %e\nT[n] %e\n",n+1,cheb0[n], cheb0[n-1]);
	a=ab_rms[0];b=ab_rms[1];
	r0=(b+a)/2;r1=(b-a)/2;
	if (!(Allroots)){
	//  pick out real roots [ -1,1]
		m=0;
		for(i=0;i<n;i++) {
			if ((wi[i]>(eps_bad_m))&&(wi[i]<eps_bad)&&(wr[i]>=eps_real_m)&&(wr[i]<=eps_real)){wi[m]=wr[i];wr[m++]=wr[i];}
		}
		nrealroots=m;
//		printf("nreal roots %d",m);
//		WAIT_CANCEL;
//		goto fin;
/* Newton Rhapson to improve roots  */
		rms=0.0;
		offset=0;
		for(i=0;i<=m;i++){
			x=wr[i];
			f0= chebx(x, cheb0, n );
			f1= chebx(x, cheb1, n );
//			clear_screen();printf("n %d\n, x %f\n f0 %e\n f1 %e\n", n,x,f0,f1);WAIT_CANCEL;
			if (fabs(f0)>eps_bad)offset++;// not considered a root, f is too far off from zero

//			else if(fabs(f1)<eps_bad) {  // possible multiple roots , cannot use NR to improve
//				wr[i-offset]=x;
//				wi[i-offset]=x;
//
//			}
			else if(NR_improve)
			{
				if(fabs(f1)>eps_bad) {dx=f0/f1;} else dx=0;// use standard NR
				x_nr=x-dx;
//				clear_screen();printf("n %d\n, x %e, xi %e", n,x,x_nr);WAIT_CANCEL;
				f1= chebx(x_nr, cheb0, n );
				if (fabs(f1)>fabs(f0))x_nr=x; // don't change x
				wr[i-offset]=x;
				wi[i-offset]=x_nr;
			}
//
		}
//		nrealroots=m-offset;

	    /* sort and remove any dublets */
		if(nrealroots>1)
		{
			sort_vector(nrealroots,wr,wi,wr);
			offset=0;m=0;
			for (i=1;i<nrealroots;i++){
				if (fabs(wi[m]-wi[i])>eps){
					++m;
					wi[m]=wi[i];
					wr[m]=wr[i];
				}
			}
			nrealroots=m+1;
			for(i=0;i<nrealroots;i++){f0=chebx(wr[i],cheb0,n+1);rms+=f0*f0;}
			rms=sqrt(rms);
			for(i=0;i<nrealroots;i++){f0=chebx(wi[i],cheb0,n+1);rms_nr+=f0*f0;}
			rms_nr=sqrt(rms_nr);
//
		}
		ab_rms[0]=rms;
		ab_rms[1]=rms_nr;
		wr=vecaddscalar(vecscale(wr, r1,nrealroots), r0, nrealroots);
		wi=vecaddscalar(vecscale(wi, r1,nrealroots), r0, nrealroots);


		if (nrealroots>0){

			sat_push_double_vector(nrealroots, wi);
			sat3_push_dbl_real(rms_nr);
			sat_push_double_vector(nrealroots, wr);
			sat3_push_dbl_real(rms);
		}
		sat3_push_int_real(nrealroots);
	}
	else
	{
		sat_push_double_cvector(n, wr,wi);
		sat3_push_int_real(n);
	}
//	free(err);
//	free(err_nr);
return (nrealroots);
}
double get_pole(double a,double b, double *T,int n){
// binary bracketing find max or min
// r_pole=get_pole(ab_rms[0],ab_rms[1], poly_input, n);
	int i;
	double mid, r_a,r_b,r ,r_mid, f;
	r_a= chebx(a,T,n);
	r_b=chebx(b,T,n);
	if ((r_a>0.0)&&(r_b>0.0)) // positive pole
	{ // estimate with double root
	mid=(a+b)/2; r_mid=chebx(mid,T,n);
	for (i=0;i<30;i++)chebx(a,T,n);
	}
	else if ((r_a<0.0)&&(r_b<0.0))  // negative pole
	{

	}
	else // pole with sign reversal
	{
// estimate with single root
		f=(1.0/f+a + b-1.0/f)/2;
	}
	return(r_mid);

}
int main(){
	const int NR_improve=TRUE;
	const int all_roots=FALSE;
    int n, file_index;
    int special=FALSE;
    int  *iterations, nrealroots;
    const double root_min_dist = 1E-6;
    double r_pole;
    double *poly,*T, *rroots,*iroots,**Coeffs, *poly_input, timings[2], ab_rms[2];
    double a,b;
    SAT_OBJECT satobj;
    int i,icol,n1,nder=0;
    tmr_t clock_pre,clock_start,clock_end,clock_post;
    int flag=0;
    char  file[2][16] ={{"chebvec0.dat"},{"chebvec1.dat"}};
    double r, input_real;

special=get_sysflag(3);
if(special) n=sat3_convertreal_int( sat3_rclhere("n"));
else      n=129;     // sample size
if (n<0){n=-n; special=0;}
if(get_userflag(1))nder=1;
/*
 *
 *
 *
 *
 */
satobj=sat3_rcl("a"); if(satobj) a=sat3_convertreal_dbl(satobj); else a=-1.0;
satobj=sat3_rcl("b"); if(satobj) b=sat3_convertreal_dbl(satobj); else b=1.0;
	if (a==b){a=-1.0;b=1.0;}

  	poly_input=create_vector(n);
  	chebsamplemain( poly_input,a,b,nder, n); // pop function in list-string format, sample and cheb transform
  	if(special) sat_push_double_vector(n,poly_input);



    /* get data from calculator stack */
//    sat3_cknerror(2);								// require two inputs on stack
//    if(sat3_checkargs(2,SAT_DOREAL, SAT_DOREAL))	// 2: vector 1:realflag
//    {
//    	input_real=sat3_pop_dbl(0.0);
//    	file_index= (int) sat3_pop_dbl(0.0);
//    	read_vector_dims_from_SD(&n,file[file_index]);
//    		poly_input=create_vector(n); read_vector_from_SD(poly_input,&n,file[file_index]);
//
////    	else
////    	{
////    		read_vector_dims_from_SD(&n,"chebvec0.dat");
////    		poly_input=create_vector(n); read_vector_from_SD(poly_input,&n,"chebvec0.dat");
////    	}
//
//    }
//    else if (sat3_checkargs(2,SAT_DOREAL, SAT_DOARRY))	// 2: vector 1:realflag
//    {
//    	input_real=sat3_pop_dbl(0.0);
//        poly_input=sat_create_pop_double_vector(&n);
//    }
//    else  sat3_exiterror(SATERR_BADARGTYPE);
    ab_rms[0]=sat3_convertreal_dbl( sat3_rclhere("a")); // read the chebyshev interval from calculator
    ab_rms[1]=sat3_convertreal_dbl( sat3_rclhere("b"));
    input_real=1.223453286;  // add root to stabilize
    if (ab_rms[0]>=ab_rms[1]){ab_rms[0]=-1.0;ab_rms[1]=1.0;} // use default a=-1.0, b=1.0
    clock_pre=tmr_ticks();
    if (ab_rms[1]-ab_rms[0]<root_min_dist){
    	/* Assume there is  1 pole
    	 * use simple bracketing to isolate pole
    	*/
    //	r_pole=get_pole(ab_rms[0],ab_rms[1], poly_input, n);
    }
    /*    optional: reduce and balance input vector when necessary and if so then increase N by 1*/
    n1=n; T=create_vector(++n1); copy_vector(T, poly_input,n); // copy to a vector large enough too include 1 additional root for stabilizing

    //    n1=8;while(n1<=n)n1*=2;n1++; T=create_vector(n1+1); copy_vector(T, poly_input,n); // copy to a vector large enough for chebyshev converting


    r=input_real; if ( r>100)r=r-100.0; if (r<-100)r=r+100;
    n1=cheb_balance(r,T, n1, &r);
//    sat_push_double_vector(n1,T);
//    clear_screen();printf("N %d r %e \n",n,r);
    n1--; // number of all roots
    r=T[n1];
//   printf("prior revscale\nN %d, r %e \n",n,r); WAIT_CANCEL;
//    sat_push_double_vector(n1+1, T);
    poly=create_vector(n1+1+1);
    for (i=0;i<=n1;i++) poly[n1+1-i]=T[i]/r; // skai reverse input, increase index by 1 due to fortran90 port
    free(T);

    //		poly_input=create_vector(N+1);
    rroots=create_vector(n1+1);
    iroots=create_vector(n1+1);
    Coeffs=create_matrix2_f(n1,3);  // matrix with option base 1, note row order not column
    iterations=create_ivector(n1+1);
    for (icol=1;icol<=n1;icol++) Coeffs[icol][1]=0.5;    // Coeffs(:,1) = .5d0
    for (icol=1;icol<=n1;icol++) Coeffs[icol][3]=0.5;    // Coeffs(:,3) = .5d0
    Coeffs[n1][1] = 1.0;
    Coeffs[n1][3] = 0.0;
    //	dnormalpoly(poly,1,10);
//    clear_screen();

//    printf("\n main r %e \n",r);

//    sat_push_double_vector(n1+1, poly+1);
    clock_start=tmr_ticks();
    flag=DAVW2(1,n1,3,poly,Coeffs,rroots-1,iroots-1,iterations-1); // index of 0 is not used
//    flag=DAVW2(1,n1,3,poly,Coeffs,rroots,iroots,iterations); // index of 0 is not used
    clock_post=tmr_ticks();
	free_matrix2_f(n1,3, Coeffs);

//if (n1>0)    sat_push_double_vector(n1, rroots);
//    sat3_push_int_real(flag);

    /*  get real results, remove duplicates and improve roots with N-R */
    nrealroots=rescheck_skai( rroots, iroots, poly_input, poly+1, n1, ab_rms, NR_improve, all_roots);
//    nrealroots=rescheck_skai( rroots+1, iroots+1, poly+1,poly_input, n1, NR_improve, all_roots);
    clock_end=tmr_ticks();
//    if(nrealroots)sat_push_double_vector(nrealroots,rroots);
//    if(nrealroots)sat_push_double_vector(nrealroots,iroots); // improved roots
//    sat3_push_int_real(nrealroots);
    timings[0]=tmr_ticks2ms(clock_start, clock_post)/1000.;
    timings[1]=tmr_ticks2ms(clock_post,clock_end)/1000.;
    sat_push_double_vector(2,timings);


    //	sat_push_double_vector(N+1+1,poly);
    //	sat_push_double_cvector(N,rroots+1,iroots+1);  // all roots
    //	sat_push_double_vector(N+1,iroots);
//    timings[0]=tmr_ticks2ms(clock_pre, clock_start)/1000.;
//    timings[1]=tmr_ticks2ms(clock_start, clock_post)/1000.;
//    timings[2]=tmr_ticks2ms(clock_post,clock_end)/1000.;
//    timings[3]=timings[0]+timings[1]+timings[2];
//	sat_push_double_vector(4,timings);
	free(poly_input);
	free(poly); // =create_vector(N+1);
	free(rroots);// =create_vector(N);
	free(iroots);// =create_vector(N);
	free(iterations); //=create_ivector(N);
	return 0;
}


BOOL get_sysflag( int sysflag){
	int nibble, bit,flag=0;
	sysflag--;   									// sysflag start at 1 , change numbering to 0
	nibble=sat3_peek(  SYSFLAG + sysflag/4, 1 );  	// read nibble  ( 4 flags)
	bit=sysflag%4; 	// 								// calculate which bit our flag is
	flag=(nibble>>bit)&1;							// pick out the flag we are looking for
/*
	if (bit==0) flag=nibble& 1;
	if (bit==1) flag=(nibble& 2)>>1;
	if (bit==2) flag=(nibble& 4)>>2;
	if (bit==3) flag=(nibble& 8)>>3;
*/
	return(flag);
}
BOOL get_userflag( int userflag){
	int nibble, bit,flag=0;
	userflag--;   									// sysflag start at 1 , change numbering to 0
	nibble=sat3_peek(  USERFLAG + userflag/4, 1 );  	// read nibble  ( 4 flags)
	bit=userflag%4; 	// 								// calculate which bit our flag is
	flag=(nibble>>bit)&1;							// pick out the flag we are looking for
	return(flag);
}

void sat_get_hpdigits(){
	int digits=0;
//	if (debug) return(10);
	digits=get_sysflag(0+HPDIGITS_FLAG)
	      | (get_sysflag(1+HPDIGITS_FLAG)<<1)
		  | (get_sysflag(2+HPDIGITS_FLAG)<<2)
		  | (get_sysflag(3+HPDIGITS_FLAG)<<3) ;
		  if(!(digits)&&(get_sysflag(FIX_ENG_FLAG)||get_sysflag(SCI_ENG_FLAG))) digits=13;
	hpdigits=digits;
}


void sat_get_mode(){
	int m;
//	if (debug) return(1);
					// mode =0  <-  DEG  <- -17 clear  -18 clear
					// mode =1  <-  RAD  <- -17 set
					// mode =2  <-  GRAD <- -17 clear  -18 set
	if (get_sysflag(RAD_FLAG)) m=1; else m=(get_sysflag(1+RAD_FLAG)<<1);
    mode=m;
}
