#include "chebroots.h"
#include "parser.h"

/*
 * parser.c
 *
 *  Created on: Jan 17, 2015
 *      Author: skai
 */


/*
 * parser.c
 *
 *  Created on: Jun 11, 2014
 *      Author: skai
 */

/*
solves 2 parameter equations in X,Y
numerical integration of 1D function,
	Romberg,
	Clenshaw-Curtis,
	doble-exponential open, ->oo,  ->oo oscilating
uses parser from Patrick Offret "Marching Cubes"
uses nelder-mead amoeba implementation from Michael F. Hutt
uses wrapper on hp 50g, expect 3:List, 2:list 1:algebraic function
 minimum version of wrapper << ROT ->STR ROT ->STR ROT  ->LST ->STR  >>
I have updated to include HPGCC3 error handling routines

 3: {a, b, c, d, I=start, stop,step } list of real values for optional  variables, can be empty list
 2: {Xinit, Yinit, eps,scale} eps and scale can be skipped, default values are eps=0.0001, scale=1, note special mode when eps=42
 1: 'f(X,Y)' recognized functions are +-*\^ SQ SQRT ABS ! LN EXP LOG ALOG SIN COS TAN ASIN ACOS ATAN SINH COSH TANH
*/

int mode;
int hpdigits;
int nf=0;
EQUATION equation;
PARAMETER parameters;
VARIABLE variable;
FUNCTION function[MAX_FUNCTION];


BOOL
sat_push_double_vector_0(int n,double *vector){
	int j;
	SAT_OBJECT sat_array;
	if (!(n))sat3_exiterror(SATERR_INVALIDDIMENSION);
    sat_array=sat3_createrarray(ALLOC_TEMPOB,n,0);
    if(sat_array==0){ sat3_exiterror( SATERR_INSUFFMEMORY ); return FALSE;}
    if (!(get_sysflag(HPSYSFLAG_TINY_ELEMENT))){
    	for (j=0;j<n;j++)if(fabs(vector[j])<1e-13)vector[j]= 0.0;}
    for (j=0;j<n;j++) sat3_rarrayput_dbl( sat_array,j,0,vector[j]);
    sat3_push(sat_array);
    return TRUE;
}

double atof_local(const char * _buf) {
    double val,expo;
    int minus,fraction_digits;
    // changed due to certain overflow conditions (excessive long strings)
    // <ibl ; 2008-04-14>
    /*ULONGLONG */ double integral,ifraction;
    char c,__buf[128],*buf;

    buf = lower(__buf,_buf); // just handle lowercase chars

    integral = ifraction = 0;
    fraction_digits = 0;
    val = 0.0;
    expo = 1.0;
//    clear_screen(); printf(buf); printf( "/n"); WAIT_CANCEL;
    if((minus = (*buf == '-'))) buf++;
    if (*buf == '+')           buf++;

    while (*buf && *buf != '.') {
        c = *buf++;
        if (isdigit(c))
        	integral = 10 * integral + c - '0';
        else {
            if (c == 'e') {buf--; goto parse_expo; }
            goto finish;
        }
    }


    if (*buf == '.') {
        buf++;
        while (*buf && *buf != 'e' ) {
            c = *buf++;
            if (! isdigit(c)) goto finish;
            fraction_digits++;
            ifraction = 10 * ifraction + c - '0';
        }
    }

parse_expo:

    if (*buf == 'e') {
        int expo_minus,ee = 0;
        buf++;
        if ((expo_minus = (*buf == '-'))) buf++;
        if (*buf == '+')    buf++;
        while (*buf && *buf !=' ') {     // skai 2009-09-05
            c = *buf++;
            ee = 10 * ee + c - '0';
        }
        expo = i10powx(expo_minus ? -ee : ee);
    }

finish:
	// REPLACED IPOW(10,X) BY I10POWX(X) - CLAUDIO MAR 2006

    val = expo * ((double) integral + (double) ifraction / i10powx(fraction_digits));

    return minus ? -val : val;
}



double convert_angle ( double s){
	static const double r[3] = { PI / 180.0 , 1.0 , PI / 200.0  };
	return (  s*r[mode]);}

double convert_angle_back(double s){
	static double rb[3] = {  180.0 / PI , 1.0 ,   200.0 / PI };
	return ( s*rb[mode]);
}


double compute_2D( double xy[]){
	double z;
	nf++;
	z = compute_xyz(xy[0], xy[1]);
	return z;
}

double compute_1D( double x){
	double z, dum=0.0;
	nf++;
	z = compute_xyz(x, dum);
	return z;
}

void replace_all_c (char *string, char old_c, char new_c){   // string must terminate with /null character
			while(!(*string=='}')){ if (*string==old_c) *string=new_c;string++;}
}


double * chebsamplemain( double *rx, double a, double b, int nder, int n) {
/*
 * CHEBSAMPLE
 * samples the function f(x) to a vector of Chebyshev coeffisients.
 * Input: f(x) as textstring in list format, 'f(x)'  -> "{ 2. X * SIN }"
 * Output: [n] ,vector of Chebyshev coeffisients
 * On the calculator run  << ->LST ->STR >> to get correct translation
 * Default input interval for x is [a .. b] = [-1..1]
 * can be changed on calculator by variables 'a' and 'b' . e.g a=-2 b=- 0.5, a<b
 * It samples at xi=  and uses FFT
 * Since it uses a variant of FFT the sample size must be n = 2^N +1
 * Result vector is pushed to stack. if calculator flag -75 (key_click ON) then saves result on SD-card
 * Standard sample size 129 if calculator flag -13 is set, then does a huge sample 1025
 */
	int i,task;
	int print_sd_mode=0, huge_sample=0;
//	SAT_OBJECT satobj;
	double eps;
	double r0,r1, err=0.0;
	double global_sum=0.0;
	print_sd_mode=(get_sysflag(75)); // save to SD only when key clicks are on
	huge_sample=(get_sysflag(13)); // if flag 13 is set , increase sample size to 1025
	sat_get_mode();
	sat_get_hpdigits();
//	satobj=sat3_rcl("a"); if(satobj) a=sat3_convertreal_dbl(satobj); else a=-1.0;
//	satobj=sat3_rcl("b"); if(satobj) b=sat3_convertreal_dbl(satobj); else b=1.0;
//	if (a==b){a=-1.0;b=1.0;}
	sat3_ckn(1);
	sat3_checkargserror(1,SAT_DOCSTR);   // exit with calculator error BADARGTYPE if not a string
	init_function();
	if(!get_equation()) sat3_exiterror(SATERR_INVALIDUSERFUNCTION); // exit with error
//	hpg_set_mode_gray16(1); hpg_clear(); hpg_flip(); hpg_clear();
	task=CHEBSAMPLE;
	switch (task){

	case CHEBSAMPLE:  // "{ X SIN }" --> :f(X):[129]  :f'(x):[129] rdynmax ,  a,b should be defined as relevant interval
//		if(huge_sample)n =1025; else
//		n= 129;
//		rx=create_vector(n);
		r0= (b-a)/2;
		r1= (b+a)/2;
		for (i=0;i<n;i++){
			rx[i]=cos(i*PI/(n-1));
			rx[i]= rx[i]*r0+r1;
			rx[i]=compute_1D(rx[i]); }
		fftini(rx, n, -5);
		rx[0]*=0.5;
		rx[n-1]*=0.5;
//		if (print_sd_mode) write_vector_to_SD(rx,n,"chebvec0.dat");
//		else sat_push_double_vector_0(n,rx);
		while ((nder--) > 0)	fftini(rx, n, 7);
//			if (print_sd_mode) write_vector_to_SD(rx,n,"chebvec1.dat");
//			else sat_push_double_vector_0(n,rx);
//		}
		return(rx);

		nf=0;
		break;


//	case CHEBS:
//		nder= 1;
//		r0= (b-a)/2;
//		r1= (b+a)/2;
//		for (i=0;i<n;i++){
//			rx[i]=cos(i*PI/(n-1));
//			rx[i]= rx[i]*r0+r1;
//			rx[i]=compute_1D(rx[i]); }
//		fftini(rx, n, -5);
//		rx[0]*=0.5;
//		rx[n-1]*=0.5;
//		while ((nder--) > 0)	fftini(rx, n, 7);
//		return(rx);
//
//		nf=0;
//		break;
//


//	case MINIMIZE2D:
//		minimum=simplex(compute_2D,start,2,parameters.epsilon,parameters.scale);
//		sat3_push_dbl_real( start[0]);
//		sat3_push_dbl_real( start[1]);
//		break;
//	case SUM2D:
//		break;
//	case INTCC1:
//	    nf=0;
//	    intcc(compute_1D, parameters.XX, parameters.YY, parameters.epsilon, lenc, c,
//	    		&global_sum, &err);
//	    printf("I= integral_0^1 4/(1+x^2) dx\n");
//	    minimum=global_sum;
//		sat3_push_int_real( nf);
//		sat3_push_dbl_real( err);
//		break;
	case INTDE:
		eps=1.0; nf=hpdigits; while ((--nf)>0)eps= eps/10.0;
		nf=0;
		intde(compute_1D, parameters.XX, parameters.YY,eps , &global_sum, &err);
		sat3_push_int_real( nf);
		sat3_push_dbl_real( err);
		sat3_push_dbl_real( global_sum);
		break;
//	case INTDEI:
//		nf=0;
//		intdei(compute_1D,parameters.XX, parameters.epsilon, &global_sum, &err);
//	    printf("I= integral_0^1 4/(1+x^2) dx\n");
//	    minimum=global_sum;
//		sat3_push_int_real( nf);
//		sat3_push_dbl_real( err);
//		break;

//		case INTDEO:
//		nf=0;
//		intdeo(compute_1D, parameters.XX, /* omega */ parameters.YY, parameters.epsilon, &global_sum, &err);
//	    minimum=global_sum;
//		sat3_push_int_real( nf);
//		sat3_push_dbl_real( err);
//		break;
//	case INTCC2D7: // Clenshaw-Curtis 2D 7x7 closed point rule, -1,1
//		nf=0;
//		cc2d7x7(compute_xyz, parameters.epsilon, &global_sum, &err);
//	    minimum=global_sum;
//		sat3_push_int_real( nf);
//		sat3_push_dbl_real( err);
//		break;
	}
	return(rx);
}



/*
 * parrom.c
 *
 *  Created on: May 7, 2014
 *      Author: skai
 */





//#define _SQRT						"\x83"
// HP50g character 131   square root
// #define PI_							"\x87"
#define SYSFLAG                      0x80F02
#define RAD_FLAG					17
// #define FIX_ENG_FLAG				49
// #define SCI_ENG_FLAG				50
// hpdigits are   flag 45-6-7-8
#define HPDIGITS_FLAG				45
// #define HPSYSFLAG_COMMA_MODE		51
// #define HPSYSFLAG_TINY_ELEMENT		54

// #define NULL 						0
// #define DEG_TO_RAD(ang) 			((ang) * PI / 180.0)
// #define EPSILON_E50					(double) (1E-50)
// #define EPSILON_E5 					(double) (1E-5)
// #define EPSILON_E3 					(double) (1E-3)

// #define pi                          PI


#define NUMBER_TYPE				0
#define FUNCTION_TYPE			1
#define VARIABLE_TYPE			2
#define RANGE_TYPE				3
#define OPERATOR_TYPE			4

#define PARSE_EQUATIONS			1
// #define FX					0
// #define FY					1
#define FZ						0

#define X_DIM					0
#define Y_DIM					1

#define	A						0
#define	B						1
#define	C						2
#define	D						3


// hpg_t *hpg_stdscreen;
// Error messages used by parser for automatic errorhandling on calculator
#define INVALID_SYNTAX     		262
#define	INVALID_USERFUNCTION   	259		// undefined function , or name not A,B,C,D
#define NOT_ENOUGH_ARM_MEMORY	 43    // buffer overflow, increase MAX_TOKEN and recompile
double convert_angle(double r);  //{ 1.0, PI / 180.0 , PI / 200.0  };
double convert_angle_back(double r); //[3]{ 1.0 , 180.0 / PI , 200.0 / PI };


double addition(), subtraction();
double multiplication(), division();
double sindeg(), cosdeg(), tandeg(), asindeg(), acosdeg(), atandeg();
double sinhp(), coshp(), tanhp();
double neg(), inv(),realsqrt();
double sq();
double exp_10();
double fact();
double piconstant();
// double simplex(double (*func)(double[]), double start[],int n_dims, double EPSILON, double scale);

double atof_local(const char * _buf);
int stack_index;
char function_name[NAME_LENGTH];
double execution_stack[STACK_DEPTH];
int Ivar=0;
void init_function(){
FUNCTION temp_function[]=
{
	{{"SIN"}, 1, {sindeg}},    //
    { { _SQRT}, 1, {realsqrt}},   // SQRT FF83 =
	{{"COS"}, 1, {cosdeg}},
	{"TAN", 1, {tandeg}},
	{{"SINH"}, 1, {sinhp}},

	{{"COSH"}, 1, {coshp}},
	{{"TANH"}, 1, {tanhp}},
	{{"ASIN"}, 1, {asindeg}},
	{{"ACOS"}, 1, {acosdeg}},
	{{"ATAN"}, 1, {atandeg}},

	{{"LN"}, 1, {log}},      // todo
	{{"LOG"}, 1, {log10}},   // todo
	{{"NEG"}, 1, {neg}},
	{{"INV"}, 1, {inv}},
	{{"ABS"}, 1, {fabs}},

	{{"EXP"}, 1, {exp}},
	{{"SQ"}, 1, {sq}},
	{{"ALOG"}, 1, {exp_10}},
	{{"!"}, 1, {fact}},
	{{"^"}, 2, {pow}},

	{{"+"}, 2, {addition}},
	{{"-"}, 2, {subtraction}},
	{{"*"}, 2, {multiplication}},
	{{"/"}, 2, {division}},
	{{ PI_ }, 0,{ piconstant}}
};
memcpy(function, temp_function, sizeof(temp_function));
//																								constants, macros, prototypes
} // end init_function
int get_equation() {
	equation.expression = sat3_pop_string_alloc();
	parse_expression();
	free(equation.expression);
	return 1;
}

int parse_expression() {
//	SAT_OBJECT sat_obj;
	int found = 0;
	double r;
	stack_index=4; // scratch for simple optimalization of constants like 2/3, f(constant) etc

	char *expression = equation.expression;

	if(*expression != '{' || expression[strlen(expression) - 1] != '}')
		sat3_exiterror(INVALID_SYNTAX); // { printf("Error: not a valid expression\n"); keyb_getkey(1); return 0;}
	expression += 2; equation.tokennumber = 0;

	while(*expression != '}') {

		if(equation.tokennumber >= MAX_TOKEN)
			sat3_exiterror(NOT_ENOUGH_ARM_MEMORY); //{printf("Error: expression too long\n"); keyb_getkey(1); return 0;}

		if( isnumber(*expression) || ( *expression == '-' &&  isnumber(* (expression + 1)))) {
			equation.token[equation.tokennumber].value = atof_local(expression);
			equation.token[equation.tokennumber++].type = NUMBER_TYPE;
			while(*expression != ' ') ++expression; ++expression;
		}

		else if( isvariable(*expression) && *(expression + 1) == ' ') {
			int index = *expression - 'a';
			r=variable.value[index];
			equation.token[equation.tokennumber].value = r;
			equation.token[equation.tokennumber++].type = NUMBER_TYPE;
//			equation.token[equation.tokennumber].index = index;
//			equation.token[equation.tokennumber++].type = VARIABLE_TYPE;
			expression += 2;
		}

		else if( isrange(*expression) && *(expression + 1) == ' ') {
			equation.token[equation.tokennumber].index = *expression - 'X'; // TODO: check if 'x'
			equation.token[equation.tokennumber++].type = RANGE_TYPE;
			expression += 2;
		}

		else if( isoperator((int) *expression) && *(expression + 1) == ' ') {
			equation.token[equation.tokennumber].index = (int) *expression;
			equation.token[equation.tokennumber++].type = OPERATOR_TYPE;
			expression += 2;
		}

		else {
			int i = 0;
			while(*expression != ' ') function_name[i++] = *expression++;
			function_name[i] = '\0'; ++expression;
//			sat_obj=sat3_rcl(function_name);
//			if (sat_obj){
//				equation.token[equation.tokennumber].value =sat3_convertreal_dbl(sat_obj);
//				equation.token[equation.tokennumber++].type = NUMBER_TYPE;
//				while(*expression != ' ') ++expression; ++expression;
//			}
//			else
//			{
				found=0;
			for(i = 0; i < MAX_FUNCTION; i++) {
				if(!strcmp(function[i].name, function_name)) {
					equation.token[equation.tokennumber].index = i;
					equation.token[equation.tokennumber++].type = FUNCTION_TYPE;
					found = 1;
//					printf ("\n found %s " ,function_name); WAIT_CANCEL;
					goto found_ok; break;
					}
				}

			if(!found) {sat3_exiterror(INVALID_USERFUNCTION);}
	//		}
		found_ok:
	// simple optimalisation, function constants and rationals
				i=equation.token[equation.tokennumber-1].index;
				execution_stack[stack_index]=equation.token[equation.tokennumber-2].value;
				execution_stack[stack_index-1]=equation.token[equation.tokennumber-3].value;
				if (
					(function[i].argnum==0)
					) {
					r=function[i].function0();
					equation.token[equation.tokennumber-1].value=r;
					equation.token[equation.tokennumber-1].type = NUMBER_TYPE; // skai
				}
				else if (
					(function[i].argnum==1) &&
					(equation.token[equation.tokennumber-2].type == NUMBER_TYPE)
					) {
					r=function[i].function1(execution_stack[stack_index]);
					equation.tokennumber--;
					equation.token[equation.tokennumber-1].value=r;
				}
				else if (
					(function[i].argnum==2) &&
					(equation.token[equation.tokennumber-2].type == NUMBER_TYPE) &&
					(equation.token[equation.tokennumber-3].type == NUMBER_TYPE)
					) {
					r=function[i].function2(execution_stack[stack_index-1],execution_stack[stack_index]);
					equation.tokennumber--;
					equation.tokennumber--;
					equation.token[equation.tokennumber-1].value=r;
				}

		}
	}
	return 1;
}
double compute_xyz( double Xvar, double Yvar) {

	stack_index = -1;
//	int xyz_index=FZ;
	int token;
	for(token = 0; token < equation.tokennumber; token++) {
		switch(equation.token[token].type) {
			case NUMBER_TYPE:	execution_stack[++stack_index] = equation.token[token].value;
								break;

			case RANGE_TYPE:	if(equation.token[token].index == X_DIM)
									execution_stack[++stack_index] = Xvar;
								else if(equation.token[token].index == Y_DIM)
										execution_stack[++stack_index] = Yvar;
								else
									execution_stack[++stack_index] = Ivar;
								break;

			case VARIABLE_TYPE:	execution_stack[++stack_index] = variable.value[equation.token[token].index];
								break;

			case OPERATOR_TYPE:	execution_stack[stack_index] = sqrt(execution_stack[stack_index]);
								break;

			case FUNCTION_TYPE:if(function[equation.token[token].index].argnum == 1)
									execution_stack[stack_index] =
										function[equation.token[token].index].function1(execution_stack[stack_index]);
								else if(function[equation.token[token].index].argnum == 2) {
									execution_stack[stack_index - 1] =
									function[equation.token[token].index].function2(execution_stack[stack_index - 1],execution_stack[stack_index]);
									--stack_index; }

								else if(function[equation.token[token].index].argnum == 3){
									execution_stack[stack_index - 2] = function[equation.token[token].index].function3(execution_stack[stack_index - 2],execution_stack[stack_index - 1],execution_stack[stack_index]);
									--stack_index;
                                    --stack_index;
								}

								else 	execution_stack[++stack_index] =function[equation.token[token].index].function0();


								break;
		}
	}
	return execution_stack[stack_index];
}


//																							math functions
double pow_skai(){ /* simple a^n implementation , n must be integer */
	int i,n;
	double a,r=1.0;
	a =execution_stack[stack_index - 1];
	n=execution_stack[stack_index];
	for (i=0;i<abs(n);i++){ r*=a;}
	if (n<0) r=1.0/r;
	return execution_stack[stack_index - 1];
} // skai todo
double addition() {return execution_stack[stack_index - 1] + execution_stack[stack_index];};
double subtraction() {return execution_stack[stack_index - 1] - execution_stack[stack_index];};
double multiplication() {return execution_stack[stack_index - 1] * execution_stack[stack_index];};
double division() {
	if(execution_stack[stack_index] == 0.0) execution_stack[stack_index] = EPSILON_E50;
	return execution_stack[stack_index - 1] / execution_stack[stack_index];
}
double neg(){return -execution_stack[stack_index];}
double inv(){if(execution_stack[stack_index] == 0.0) execution_stack[stack_index] = EPSILON_E50;return 1.0 / execution_stack[stack_index];}
double realsqrt(){ return (execution_stack[stack_index]>=0.0) ? sqrt(execution_stack[stack_index]): (sat3_exiterror(SATERR_BADARGVALUE), 0.0);}
double sq() { return execution_stack[stack_index] * execution_stack[stack_index];};
double sindeg(){ return sin(convert_angle(execution_stack[stack_index]));}
double cosdeg(){ return cos(convert_angle(execution_stack[stack_index]));}
double tandeg(){ return tan(convert_angle(execution_stack[stack_index]));}
double asindeg(){ return (execution_stack[stack_index]>=-1.0&&execution_stack[stack_index]<= 1.0)
				? convert_angle_back(asin(execution_stack[stack_index])) : (  sat3_exiterror(SATERR_BADARGVALUE), 0.0);}
double acosdeg(){ return (execution_stack[stack_index]>=-1.0&&execution_stack[stack_index]<= 1.0)
		? convert_angle_back(acos(execution_stack[stack_index])) : (  sat3_exiterror(SATERR_BADARGVALUE), 0.0);}
double atandeg(){ return convert_angle_back(atan(execution_stack[stack_index]));}
double sinhp(){return (exp(execution_stack[stack_index]) - exp(-execution_stack[stack_index])) * 0.5;}
double coshp(){return (exp(execution_stack[stack_index]) + exp(-execution_stack[stack_index])) * 0.5;}
double tanhp(){double q; if( (q = coshp()) == 0.0) q = EPSILON_E5; return (sinhp() / q);}
double exp_10(){ return (exp(LOG10*execution_stack[stack_index]));}
double fact(){ int i,i2; double rfact=1; i2=execution_stack[stack_index]; for (i=1; i<=i2;i++)rfact*=i; return (rfact);}
double piconstant() {return ( PI );}



// void dfct(int, double *);
//void dfst(int, double *);
//double * der1(int nm1, double * a, int bigrange, double aa,double bb);
// void putdata(int nini, int nend, double *a);
// double errorcheck(int nini, int nend, double scale, double *a);
// void scalevec(double *a, double f, int n0, int n1);

void scalevec(double *a, double f, int n0, int n1){
	int j;
    for (j = n0; j < n1; j++) a[j] *= f;
}
//double * der1(int nm1, double * a, int bigrange, double aa,double bb){ // TODO: verify
//// used by cos transform (due to task 5)
//	// ref aaabook 9.18
//	// nm1:  a[0..n]  n dim of a, nm1=n-1,
//	int i;
//	double t0,t1=0.0,t2=0.0, r;
//	r=a[nm1];
//	a[nm1]=0.0;
//	 for(i=nm1;i>0;i--){
//		 t0=r*i*2+t2; //
//		 t2=t1;t1=t0;
//		 r=a[i-1];
//		 a[i-1]=t0;
//	 }
//	    a[0]*=.5; // check this in connection to bigrange
//    if(bigrange) scalevec(a,2.0/(bb-aa), 0,nm1);
//	return a;
//}

int fftini(  double *a, int n, int task ){
    int n2=1, nm1;
    n--; do {n2*=2;}while ((n2<n)); if (n2>n) { sat3_exiterror (SATERR_INVALIDDIMENSION);}
    n++;
    nm1=n-1;
    if (task==5) { a[0] *= 0.5; a[nm1] *= 0.5;  /* skai */ dfct(nm1, a); /* skai */ }/* Cosine Transform of RDFT (Real Symmetric DFT)   */
    else if (task==-5) { a[0] *= 0.5; a[nm1] *= 0.5; dfct(nm1, a); scalevec(a,2.0/nm1, 0,n); } /* inverse Cosine Transform of RDFT (Real Symmetric DFT)   */ //      for (j = 0; j <= nm1; j++) {a[j] *= 2.0 / nm1;}
    /* check of DFST */
//    else if (task==6) 	dfst(n, a);  /* Sine Transform of RDFT (Real Anti-symmetric DFT)   */
//    else if (task==-6) { dfst(n, a);scalevec(a,2.0/n,1, n); } /* inverse Sine Transform of RDFT (Real Anti-symmetric DFT)   */ //    for (j = 1; j <= n - 1; j++) a[j] *= 2.0 / n;

    else if (task==7){   der1(nm1, a, 0, -1.0, 1.0);}
//    else if (task==-7){ a[0] *= 0.5;
//    					a[nm1] *= 0.5;
//    					dfct(nm1, a);
//    					scalevec(a,2.0/nm1, 0,n); // inverse Cos
//    					a[0] *= 0.5; a[nm1] *= 0.5;
//    					der1(nm1, a, 0, -1.0, 1.0); 	  	  	  	  // deriver once
//    					dfct(nm1, a); // cos transform
//
//    	 }
//    else if (task==-8){ a[0] *= 0.5;
//    					a[nm1] *= 0.5;
//    					dfct(nm1, a);
//    					scalevec(a,2.0/nm1, 0,n); // inverse Cos
//    					a[0] *= 0.5; a[nm1] *= 0.5;
//    					der1(nm1, a, 0, -1.0, 1.0);								// deriver once
//    					der1(nm1, a, 0, -1.0, 1.0);								// deriver twice
//    					dfct(nm1, a); // cos transform
//
//    	 }

return 0;
}
