// #ifndef PARSER_H_
// #define PARSER_H_
#include <hpgcc3.h>
// #include <hpgraphics.h>


#define MAX_TOKEN				255
#define MAX_VARIABLE			4
#define MAX_PARAMETER			5

#define MAX_RANGE				3
#define MAX_FUNCTION			25

#define STACK_DEPTH				255
#define NAME_LENGTH				20

#define Too_Few_Arguments 513	// #201 err number in HP
#define Bad_Argument_Type 514	// #202 err number in HP
#define Invalid_Syntax 262	    // #106 err number in HP
#define Positive_Underflow 769 	// #301 err number in HP
#define Negative_Underflow 770	// #302 err number in HP
#define Overflow 771			// #303 err number in HP
#define Undefined_Result 772  	// #304 err number in HP
#define Infinite_Result 773  	// #305 err number in HP
#define Invalid_Dimensions 1281	// #501 err number in HP
// Error messages used by parser for automatic errorhandling on calculator
#define INVALID_SYNTAX     		262
#define	INVALID_USERFUNCTION   	259		// undefined function , or name not A,B,C,D
#define NOT_ENOUGH_ARM_MEMORY	 43    // buffer overflow, increase MAX_TOKEN and recompile






#define _SQRT						"\x83"
// HP50g character 131   square root
#define PI_							"\x87"
#define SYSFLAG                      0x80F02
#define	RAD_FLAG					17
#define FIX_ENG_FLAG				49
#define SCI_ENG_FLAG				50
#define PSEUDO_INFINITE             ((double) .1E309)
// hpdigits are   flag 45-6-7-8
#define HPDIGITS_FLAG				45
#define HPSYSFLAG_COMMA_MODE		51
#define HPSYSFLAG_TINY_ELEMENT		54

#define NULL 						0
// #define PI         					((double) 3.14159265358979)
#define DEG_TO_RAD(ang) 			((ang) * PI / 180.0)
#define EPSILON_E50					(double) (1E-50)
#define EPSILON_E5 					(double) (1E-5)
#define EPSILON_E3 					(double) (1E-3)

// #define pi                          PI
#define LOG10						((double) 2.30258509299406)
#define HUGE						((double)1E300)
#define PARSETEST	  			0
#define CHEBSAMPLE 	 			1
// generate gauss cheb and sample
#define VECTORSAMPLE 	 		2
#define CHEBSAMPLE3 	 		3
// generate gauss cheb and sample, use  interval a..b, include high dynamic warning
// #define MINIMIZE2D  			2
//#define SUM2D		  			3
#define INTCC1 	 				4
#define INTDE					5 // integrator of f(x) over (a,b).
#define INTDEI					6 // integrator of f(x) over (a,infinity),f(x) is non oscillatory function.
#define INTDEO					7 // integrator of f(x) over (a,infinity),f(x) is oscillatory function.
#define INTCC2D7 				8 // Clenshaw-Curtis 2D 7x7 closed point rule, -1,1

#define isdigit(c)  			(c >= '0' && c <= '9')

#define isnumber(c)  			((c >= '0' && c <= '9') || c == '.' )
#define isvariable(c)  			(c == 'a' || c == 'b' || c == 'c' || c == 'd' )
// #define ispiconstant(c)			(c == 135 )

#define isrange(c)				(c == 'X' || c == 'Y' || c == 'x' || c == 'y'  )
#define isoperator(c)			(c == 131)



typedef struct {
	union {
		struct {
			double XX, YY, type, epsilon, scale;};
		double p[5];
	};
} PARAMETER;

typedef struct {
	struct {
		double a, b, c, d;};
	double value[MAX_VARIABLE];

} VARIABLE;

typedef struct {
	int type;
	union {
		double value;
		int index;
	};
} TOKEN;

typedef struct {
	char name[20];
	int argnum;
	union{
	   double (*function0)();
	   double (*function1)(double x1);
	   double (*function2)(double x1,double x2);
	   double (*function3)(double x1,double x2, double x3);
     };
} FUNCTION;

typedef struct {
	char *expression;
	char *parameter;
	char *variable;
	TOKEN token[MAX_TOKEN];
	int tokennumber;
} EQUATION;
//int hpdigits;
//int mode;
void sat_get_hpdigits();
void sat_get_mode();
BOOL get_sysflag( int sysflag);
// double simplex(double (*func)(double[]), double start[],int n, double EPSILON, double scale);
void init_function();
double compute_2D( double xy[]);
double compute_1D( double x);
double compute_xyz(double x, double y);

// double ROMINT ( double *val, double *err, double romint_eps,
//		double a, double b, int n, double (*func)(double x));
// void intcc(double (*f)(double), double a, double b, double eps,
//    int lenc, double *c, double *i, double *err);
void intde(double (*f)(double), double a, double b, double eps,
    double *i, double *err);
// void intdei(double (*f)(double), double a, double eps, double *i, double *err);
// void intdeo(double (*f)(double), double a, double omega, double eps,    double *i, double *err);
// void cc2d7x7(double (*f)(double xdum, double ydum), double eps,
//	    double *intsum, double *err);
//void dfct(int, double *);
void scalevec(double *a, double f, int n0, int n1);

// int write_vector_to_SD(double *a, int n, char* name  );
// int read_vector_from_SD(double *a, int *n, char* name  );
// int read_vector_dims_from_SD(int *n, char* name  );

BOOL sat_push_double_vector_0(int elements,double *vector);
double *create_vector(int m);
void const_vector(int n, double r, double *v);
//int fftini(  double *a, int n, int task );
void replace_all_c (char *string, char old_c, char new_c);
// void sat_get_mode();
double convert_angle_back(double s);
// BOOL get_sysflag( int sysflag);


int parse_expression();
// int parse_expression(int xyz_index, FUNCTION function[MAX_FUNCTION]);

void get_parameters();
int get_equation();
//extern int  mode;
//extern int hpdigits;
//extern EQUATION equation;
//extern PARAMETER parameters;
//
//// X0, Y0, type, epsilon, scale
//extern VARIABLE variable;

// #endif /* PARROM_H_ */
